-- Package declaration
local _nythendra = _G["github.com/eandre/groupauras/auras/emerald_nightmare/nythendra"] or {}
_G["github.com/eandre/groupauras/auras/emerald_nightmare/nythendra"] = _nythendra

local builtins = _G.lunar_go_builtins

local _classutil = _G["github.com/eandre/groupauras/pkg/classutil"]
local _context = _G["github.com/eandre/groupauras/pkg/context"]
local _draw = _G["github.com/eandre/groupauras/pkg/draw"]
local _raidutil = _G["github.com/eandre/groupauras/pkg/raidutil"]
local _auratrack = _G["github.com/eandre/groupauras/pkg/raidutil/auratrack"]
local _inspect = _G["github.com/eandre/groupauras/pkg/raidutil/inspect"]
local _rdu = _G["github.com/eandre/groupauras/pkg/raidutil/rdu"]
local _wow = _G["github.com/eandre/lunar-wow/pkg/wow"]

_nythendra.InfestedMindID = 205043
_nythendra.InfestedMindInterrupters = 2
_nythendra.InfestedMindInterruptRange = 10
_nythendra.VolatileRotID = 204463

_nythendra.Nythendra = {}

_nythendra.Nythendra._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_nythendra.Nythendra})
	local obj, err
	

		obj, err = _auratrack.Tracker._createFromTable(tbl.InfestedMind)
		if err ~= nil then
			return nil, err
		end
		self.InfestedMind = obj
					

		obj, err = _auratrack.Tracker._createFromTable(tbl.VolatileRot)
		if err ~= nil then
			return nil, err
		end
		self.VolatileRot = obj
					
	self.dt = 0
	if type(self.dt) == type(tbl.dt) then self.dt = tbl.dt end
	return self, nil
end

_nythendra.Nythendra._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		

		obj, err = _auratrack.Tracker._createFromTable(tbl.InfestedMind)
		if err ~= nil then
			return err
		end
		self.InfestedMind = obj
					

		obj, err = _auratrack.Tracker._createFromTable(tbl.VolatileRot)
		if err ~= nil then
			return err
		end
		self.VolatileRot = obj
					
	if type(self.dt) == type(tbl.dt) then self.dt = tbl.dt end
	return nil
end

_nythendra.Nythendra.updateInfestedMind = function(n, dt)
	n.dt = n.dt + dt
	if n.dt < 0.1 then
		return
	end
	n.dt = n.dt - 0.1
	for _, target in pairs(n.InfestedMind.Targets or {}) do
		local close = _raidutil.GroupMembersWithin(target.GUID, _nythendra.InfestedMindInterruptRange)
		local interrupters = 0
		for _, c in builtins.slice_iter(close or {}) do
			if (n.InfestedMind.Targets[c.GUID] or nil) == nil then
				if _nythendra.hasInterrupt(c.GUID) then
					interrupters = interrupters + 1
				end
			end
		end
		local pt = target.Data
		if interrupters >= _nythendra.InfestedMindInterrupters then
			pt:SetVertexColor(0.25, 1, 0.25, 0.5)
		else
			pt:SetVertexColor(1, 0.25, 0.25, 0.5)
		end
	end
end

_nythendra.Enable = function(ctx)
	local n = setmetatable({ ["Ctx"] = ctx, ["dt"] = 0 }, {__index=_nythendra.Nythendra})
	n.InfestedMind = _auratrack.New(ctx, _nythendra.InfestedMindID, { ["Add"] = function(target)
		local pt = _draw.NewPoint({ ["Ctx"] = target.Ctx, ["Pos"] = _draw.GUIDPosition(target.GUID), ["Text"] = target.Name, ["Texture"] = "highlight", ["SizeYards"] = _nythendra.InfestedMindInterruptRange, ["SizePixels"] = 0, ["RotateDegrees"] = 0, ["RotateSpeed"] = 0, ["DrawLayer"] = "", ["DrawOrder"] = 0})
		target.Data = pt
		_rdu.ShowGroupMembers(target.Ctx)
		if builtins.mapLength(n.InfestedMind.Targets) == 1 then
			_wow.RegisterUpdate(builtins.create_closure(n, "updateInfestedMind"))
		end
	end, ["Remove"] = function(target)
		if builtins.mapLength(n.InfestedMind.Targets) == 0 then
			_wow.UnregisterUpdate(builtins.create_closure(n, "updateInfestedMind"))
		end
	end})
	n.VolatileRot = _auratrack.New(ctx, _nythendra.VolatileRotID, { ["Add"] = function(target)
		_draw.NewPoint({ ["Ctx"] = target.Ctx, ["Pos"] = _draw.GUIDPosition(target.GUID), ["Texture"] = "falloffcircle", ["VertexColor"] = { 1, 0.25, 0.25, 0.5 }, ["SizeYards"] = 50, ["Text"] = "", ["SizePixels"] = 0, ["RotateDegrees"] = 0, ["RotateSpeed"] = 0, ["DrawLayer"] = "", ["DrawOrder"] = 0})
		_rdu.ShowUnit(target.Ctx, target.GUID)
	end})
	return n
end

_nythendra.hasInterrupt = function(guid)
	local data, ok = _inspect.DataForGUID(guid)
	if (not ok) then
		return false
	end
	local spec = (_classutil.Specs[data.SpecID] or nil)
	if spec == nil then
		print("groupauras: No spec found for id", data.SpecID)
		return false
	end
	return spec.Melee and spec.HasInterrupt
end

