-- Package declaration
local _aura = _G["github.com/eandre/groupauras/core/aura"] or {}
_G["github.com/eandre/groupauras/core/aura"] = _aura

local builtins = _G.lunar_go_builtins

local _luautil = _G["github.com/eandre/lunar-wow/pkg/luautil"]

_aura.CompiledAura = {}

_aura.CompiledAura._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_aura.CompiledAura})
	local obj, err
	

		obj, err = _aura.Aura._createFromTable(tbl.Aura)
		if err ~= nil then
			return nil, err
		end
		self.Aura = obj
					
	self.Enables = nil
	if type(self.Enables) == type(tbl.Enables) then self.Enables = tbl.Enables end
	self.Disables = nil
	if type(self.Disables) == type(tbl.Disables) then self.Disables = tbl.Disables end
	self.Events = nil
	if type(self.Events) == type(tbl.Events) then self.Events = tbl.Events end
	self.OnUpdate = nil
	if type(self.OnUpdate) == type(tbl.OnUpdate) then self.OnUpdate = tbl.OnUpdate end
	self.OnActivate = nil
	if type(self.OnActivate) == type(tbl.OnActivate) then self.OnActivate = tbl.OnActivate end
	self.OnDeactivate = nil
	if type(self.OnDeactivate) == type(tbl.OnDeactivate) then self.OnDeactivate = tbl.OnDeactivate end
	return self, nil
end

_aura.CompiledAura._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		

		obj, err = _aura.Aura._createFromTable(tbl.Aura)
		if err ~= nil then
			return err
		end
		self.Aura = obj
					
	if type(self.Enables) == type(tbl.Enables) then self.Enables = tbl.Enables end
	if type(self.Disables) == type(tbl.Disables) then self.Disables = tbl.Disables end
	if type(self.Events) == type(tbl.Events) then self.Events = tbl.Events end
	if type(self.OnUpdate) == type(tbl.OnUpdate) then self.OnUpdate = tbl.OnUpdate end
	if type(self.OnActivate) == type(tbl.OnActivate) then self.OnActivate = tbl.OnActivate end
	if type(self.OnDeactivate) == type(tbl.OnDeactivate) then self.OnDeactivate = tbl.OnDeactivate end
	return nil
end

_aura.Compile = function(rev)
	local ca = _aura.newCompiledAura(rev)
	for event, src in pairs(rev.Enables or {}) do
		local f, err = _luautil.Eval(src)
		if err ~= nil then
			return nil, err
		end
		ca.Enables[event] = f
	end
	for event, src in pairs(rev.Disables or {}) do
		local f, err = _luautil.Eval(src)
		if err ~= nil then
			return nil, err
		end
		ca.Disables[event] = f
	end
	for event, src in pairs(rev.Events or {}) do
		local f, err = _luautil.Eval(src)
		if err ~= nil then
			return nil, err
		end
		ca.Events[event] = f
	end
	if rev.OnUpdate ~= "" then
		local f, err = _luautil.Eval(rev.OnUpdate)
		if err ~= nil then
			return nil, err
		end
		ca.OnUpdate = f
	end
	if rev.OnActivate ~= "" then
		local f, err = _luautil.Eval(rev.OnActivate)
		if err ~= nil then
			return nil, err
		end
		ca.OnActivate = f
	end
	if rev.OnDeactivate ~= "" then
		local f, err = _luautil.Eval(rev.OnDeactivate)
		if err ~= nil then
			return nil, err
		end
		ca.OnDeactivate = f
	end
	return ca, nil
end

_aura.newCompiledAura = function(rev)
	return { ["Aura"] = rev, ["Enables"] = {}, ["Disables"] = {}, ["Events"] = {}}
end

