-- Package declaration
local _watcher = _G["github.com/eandre/groupauras/core/watcher"] or {}
_G["github.com/eandre/groupauras/core/watcher"] = _watcher

local builtins = _G.lunar_go_builtins

local _aura = _G["github.com/eandre/groupauras/core/aura"]
local _library = _G["github.com/eandre/groupauras/core/library"]
local _runner = _G["github.com/eandre/groupauras/core/runner"]
local _wow = _G["github.com/eandre/lunar-wow/pkg/wow"]

_watcher.Watcher = {}

_watcher.Watcher._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_watcher.Watcher})
	local obj, err
	
	self.all = nil
	if type(self.all) == type(tbl.all) then self.all = tbl.all end
	self.enables = nil
	if type(self.enables) == type(tbl.enables) then self.enables = tbl.enables end
	self.disables = nil
	if type(self.disables) == type(tbl.disables) then self.disables = tbl.disables end
	self.listeners = nil
	if type(self.listeners) == type(tbl.listeners) then self.listeners = tbl.listeners end

		obj, err = _runner.Runner._createFromTable(tbl.runner)
		if err ~= nil then
			return nil, err
		end
		self.runner = obj
					
	return self, nil
end

_watcher.Watcher._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		
	if type(self.all) == type(tbl.all) then self.all = tbl.all end
	if type(self.enables) == type(tbl.enables) then self.enables = tbl.enables end
	if type(self.disables) == type(tbl.disables) then self.disables = tbl.disables end
	if type(self.listeners) == type(tbl.listeners) then self.listeners = tbl.listeners end

		obj, err = _runner.Runner._createFromTable(tbl.runner)
		if err ~= nil then
			return err
		end
		self.runner = obj
					
	return nil
end

_watcher.New = function(l, r)
	local w = setmetatable({ ["all"] = {}, ["enables"] = {}, ["disables"] = {}, ["listeners"] = {}, ["runner"] = r }, {__index=_watcher.Watcher})
	l:RegisterListener(w)
	return w
end

_watcher.Watcher.OnAuraChanged = function(w, a, enabled)
	local prev = (w.all[a.ID] or nil)
	if prev ~= nil then
		w:removeAura(prev)
	end
	if (not enabled) then
		return
	end
	local ca, err = _aura.Compile(a)
	if err ~= nil then
		print("Could not compile aura:", err:Error())
		return
	end
	w:addAura(ca)
end

_watcher.Watcher.addAura = function(w, ca)
	if (w.all[ca.Aura.ID] or nil) ~= nil then
		return
	end
	w.all[ca.Aura.ID] = ca
	for event, f in pairs(ca.Enables or {}) do
		local m = (w.enables[event] or nil)
		if m == nil then
			m = {}
			w.enables[event] = m
		end
		m[ca] = f
		w:addListener(event)
	end
	for event, f in pairs(ca.Disables or {}) do
		local m = (w.disables[event] or nil)
		if m == nil then
			m = {}
			w.disables[event] = m
		end
		m[ca] = f
		w:addListener(event)
	end
end

_watcher.Watcher.removeAura = function(w, ca)
	if (w.all[ca.Aura.ID] or nil) == nil then
		return
	end
	builtins.delete(w.all, ca.Aura.ID)
	for event in pairs(ca.Enables or {}) do
		local m = (w.enables[event] or nil)
		builtins.delete(m, ca)
		if builtins.mapLength(m) == 0 then
			builtins.delete(w.enables, event)
		end
		w:removeListener(event)
	end
	for event in pairs(ca.Disables or {}) do
		local m = (w.disables[event] or nil)
		builtins.delete(m, ca)
		if builtins.mapLength(m) == 0 then
			builtins.delete(w.disables, event)
		end
		w:removeListener(event)
	end
end

_watcher.Watcher.addListener = function(w, event)
	local n = (w.listeners[event] or 0)
	w.listeners[event] = n + 1
	if n == 0 then
		_wow.RegisterEvent(event, builtins.create_closure(w, "onEvent"))
	end
end

_watcher.Watcher.removeListener = function(w, event)
	local n = (w.listeners[event] or 0)
	w.listeners[event] = n - 1
	if n == 1 then
		builtins.delete(w.listeners, event)
		_wow.UnregisterEvent(event, builtins.create_closure(w, "onEvent"))
	end
end

_watcher.Watcher.onEvent = function(w, event, args)
	for ca, f in pairs((w.enables[event] or nil) or {}) do
		if (not w.runner:Running(ca)) and f(ca, event, args) then
			w.runner:Start(ca)
		end
	end
	for ca, f in pairs((w.disables[event] or nil) or {}) do
		if w.runner:Running(ca) and f(ca, event, args) then
			w.runner:Stop(ca)
		end
	end
end

