-- Package declaration
local _context = _G["github.com/eandre/groupauras/pkg/context"] or {}
_G["github.com/eandre/groupauras/pkg/context"] = _context

local builtins = _G.lunar_go_builtins



_context.callback = {}

_context.callback._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_context.callback})
	local obj, err
	
	self.f = nil
	if type(self.f) == type(tbl.f) then self.f = tbl.f end
	return self, nil
end

_context.callback._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		
	if type(self.f) == type(tbl.f) then self.f = tbl.f end
	return nil
end

_context.ctx = {}

_context.ctx._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_context.ctx})
	local obj, err
	
	self.children = nil
	if type(self.children) == type(tbl.children) then self.children = tbl.children end
	self.callbacks = nil
	if type(self.callbacks) == type(tbl.callbacks) then self.callbacks = tbl.callbacks end
	self.cancelled = false
	if type(self.cancelled) == type(tbl.cancelled) then self.cancelled = tbl.cancelled end
	return self, nil
end

_context.ctx._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		
	if type(self.children) == type(tbl.children) then self.children = tbl.children end
	if type(self.callbacks) == type(tbl.callbacks) then self.callbacks = tbl.callbacks end
	if type(self.cancelled) == type(tbl.cancelled) then self.cancelled = tbl.cancelled end
	return nil
end

_context.ctx.cancel = function(ctx)
	if ctx.cancelled then
		return
	end
	if ctx.parent ~= nil then
		ctx.parent:removeChild(ctx)
	end
	for child in pairs(ctx.children or {}) do
		child:cancel()
	end
	for _, cb in builtins.slice_iter(ctx.callbacks or {}) do
		cb.f(ctx, cb.data)
	end
	ctx.cancelled = true
end

_context.ctx.OnCancel = function(ctx, data, f)
	if ctx.cancelled then
		f(ctx, data)
		return
	end
	ctx.callbacks = builtins.append(ctx.callbacks, { ["f"] = f, ["data"] = data})
end

_context.ctx.Cancelled = function(ctx)
	return ctx.cancelled
end

_context.ctx.addChild = function(ctx, child)
	ctx.children[child] = true
end

_context.ctx.removeChild = function(ctx, child)
	builtins.delete(ctx.children, child)
end

_context.New = function(parent)
	local c = setmetatable({ ["parent"] = parent, ["children"] = {}, ["cancelled"] = false }, {__index=_context.ctx})
	parent:addChild(c)
	return c, builtins.create_closure(c, "cancel")
end

_context.baseCtx = {}

_context.baseCtx._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_context.baseCtx})
	local obj, err
	
	return self, nil
end

_context.baseCtx._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		
	return nil
end

_context.baseCtx.OnCancel = function(ctx, data, callback)
end

_context.baseCtx.Cancelled = function(ctx)
	return false
end

_context.baseCtx.cancel = function(ctx)
end

_context.baseCtx.addChild = function(ctx, child)
end

_context.baseCtx.removeChild = function(ctx, child)
end

_context.Base = setmetatable({  }, {__index=_context.baseCtx})

