-- Package declaration
local _raidutil = _G["github.com/eandre/groupauras/pkg/raidutil"] or {}
_G["github.com/eandre/groupauras/pkg/raidutil"] = _raidutil

local builtins = _G.lunar_go_builtins

local _hbd = _G["github.com/eandre/lunar-shim/hbd"]
local _luastrings = _G["github.com/eandre/lunar-wow/pkg/luastrings"]
local _luatable = _G["github.com/eandre/lunar-wow/pkg/luatable"]
local _wow = _G["github.com/eandre/lunar-wow/pkg/wow"]

_raidutil.RangeResult = {}

_raidutil.RangeResult._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_raidutil.RangeResult})
	local obj, err
	
	self.UnitID = ""
	if type(self.UnitID) == type(tbl.UnitID) then self.UnitID = tbl.UnitID end
	self.GUID = ""
	if type(self.GUID) == type(tbl.GUID) then self.GUID = tbl.GUID end
	self.Yards = 0
	if type(self.Yards) == type(tbl.Yards) then self.Yards = tbl.Yards end
	return self, nil
end

_raidutil.RangeResult._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		
	if type(self.UnitID) == type(tbl.UnitID) then self.UnitID = tbl.UnitID end
	if type(self.GUID) == type(tbl.GUID) then self.GUID = tbl.GUID end
	if type(self.Yards) == type(tbl.Yards) then self.Yards = tbl.Yards end
	return nil
end

_raidutil.ranges = {}

_raidutil.GroupMembersWithin = function(guid, yards)
	local result = nil
	
	local r = (_raidutil.ranges[guid] or nil)
	if builtins.length(r) == 0 then
		return nil
	end
	for _, res in builtins.slice_iter(r or {}) do
		if res.Yards <= yards then
			result = builtins.append(result, res)
		end
	end
	return result
end

_raidutil.updateRanges = function()
	_luatable.Wipe(_raidutil.ranges)
	local inRaid = _wow.IsInRaid()
	local prefix = "party"
	if inRaid then
		prefix = "raid"
	end
	local num = _wow.GetNumGroupMembers()
	do
		local i = 1
		while i <= num do
			local src = (prefix .. _luastrings.ToString(i))
			if (not inRaid) and i == num then
				src = "player"
			end
			local srcGUID = _wow.UnitGUID(src)
			local srcX, srcY, srcInst = _hbd.UnitWorldPosition(src)
			local result = nil
			
			do
				local j = 1
				while j < num do
					local dst = (prefix .. _luastrings.ToString(j))
					local dstX, dstY, dstInst = _hbd.UnitWorldPosition(dst)
					if dstInst == srcInst then
						local yards = _hbd.WorldDistance(srcInst, srcX, srcY, dstX, dstY)
						local dstGUID = _wow.UnitGUID(dst)
						result = builtins.append(result, { ["UnitID"] = dst, ["GUID"] = dstGUID, ["Yards"] = yards})
						_raidutil.ranges[dstGUID] = builtins.append((_raidutil.ranges[dstGUID] or nil), { ["UnitID"] = src, ["GUID"] = srcGUID, ["Yards"] = yards})
					end
					j = j + 1
				end
			end
			_raidutil.ranges[srcGUID] = result
			i = i + 1
		end
	end
end

_raidutil.elapsed = 0

_raidutil.onUpdate = function(dt)
	_raidutil.elapsed = _raidutil.elapsed + dt
	if _raidutil.elapsed < 0.1 then
		return
	end
	_raidutil.elapsed = _raidutil.elapsed - 0.1
	_raidutil.updateRanges()
end

builtins.add_init(function()
	_wow.RegisterUpdate(_raidutil.onUpdate)
end)

